/* gimpbuttons.c
 * gimpbuttons: an extension for the GIMP that does some stuff.
 * by Simon Budig <Simon.Budig@unix-ag.org>
 */

/* The GIMP -- an image manipulation program
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Code blatantly stolen from Kevin's janitor.c who himself added the notice: */

/* Code blatantly stolen from Sven's screenshot.c */

/* I love GPL... :-) (Simon) */

#include <gtk/gtk.h>
#include <libgimp/gimp.h>
#include <stdio.h>
#include <stdlib.h>

/* Defines */
#define PLUG_IN_NAME        "extension_gimpbuttons"
#define PLUG_IN_PRINT_NAME  "Buttons"
#define PLUG_IN_VERSION     "0.1.3"
#define PLUG_IN_MENU_PATH   "<Toolbox>/Xtns/Buttons"
#define PLUG_IN_AUTHOR      "Simon Budig <Simon.Budig@unix-ag.org>"
#define PLUG_IN_COPYRIGHT   "Simon Budig"
#define PLUG_IN_DESCRIBTION "Shows some Buttons, which can be configured to some PDB-Calls via ~/.gimp/gimpbuttonrc."
#define PLUG_IN_HELP        "FIXME: Write help text."

#define NUMBER_IN_ARGS 1
#define MAX_BUTTONS 20
#define MAX_ARGS 40
#define IN_ARGS { PARAM_INT32,    "run_mode", "Interactive, non-interactive" }

#define NUMBER_OUT_ARGS 0

typedef struct {
  guchar *name;
  guchar *buttontext;
  gint nparam;
  GParam *parameters;
  GtkWidget *button;
} ProcEntry;
  
static void  query (void);
static void  run (gchar *name,
		  gint nparams,	          /* number of parameters passed in */
		  GParam * param,	  /* parameters passed in */
		  gint *nreturn_vals,     /* number of parameters returned */
		  GParam ** return_vals); /* parameters to be returned */
void scan();
static gint  buttons_dialog (void);
static void  dialog_close_callback (GtkWidget *widget,
				   gpointer   data);
static void button_clicked_callback (GtkWidget *widget, gpointer data);

/* Global Variables */
GPlugInInfo PLUG_IN_INFO =
{
  NULL,   /* init_proc  */
  NULL,	  /* quit_proc  */
  query,  /* query_proc */
  run     /* run_proc   */
};

ProcEntry Buttons[MAX_BUTTONS];

gint ActiveImage=-1;
  
/* Functions */

MAIN ()

static void query (void)
{

static GParamDef args[] = { IN_ARGS };
static gint nargs = NUMBER_IN_ARGS;
/* static GParamDef return_vals[] = { NULL };  */
static gint nreturn_vals = NUMBER_OUT_ARGS; 

/* the actual installation of the plugin */
gimp_install_procedure (PLUG_IN_NAME,
			PLUG_IN_DESCRIBTION,
			PLUG_IN_HELP,
			PLUG_IN_AUTHOR,
			PLUG_IN_COPYRIGHT,
			PLUG_IN_VERSION,
			PLUG_IN_MENU_PATH,
			NULL,
			PROC_EXTENSION,		
			nargs,
			nreturn_vals,
			args,
		        NULL /*return_vals */);
}

static void 
run (gchar *name,		/* name of plugin */
     gint nparams,		/* number of in-paramters */
     GParam * param,		/* in-parameters */
     gint *nreturn_vals,	/* number of out-parameters */
     GParam ** return_vals)	/* out-parameters */
{

  /* Get the runmode from the in-parameters */
  GRunModeType run_mode = param[0].data.d_int32;	
  
  /* status variable, use it to check for errors in invocation usualy only 
     during non-interactive calling */	
  GStatusType status = STATUS_SUCCESS;	 
  
  /*always return at least the status to the caller. */
  static GParam values[1];
  
  /* initialize the return of the status */ 	
  values[0].type = PARAM_STATUS;
  values[0].data.d_status = status;
  *nreturn_vals = 1;
  *return_vals = values;
  
  /* how do we want to run today? */
  switch (run_mode)
  {
    case RUN_INTERACTIVE:
      /* Get information from the dialog */
      scan();
      buttons_dialog();
      status = STATUS_SUCCESS;
      break;

    case RUN_NONINTERACTIVE:
      /* right now, I don't want to handle non-interactive calls */
      status = STATUS_CALLING_ERROR;
      break;
      
    default:
      break;
  } /* switch */

  /* The dialog is *it*, hun.  We can go now. */

  values[0].data.d_status = status; 
}


void scan() {
   FILE *cf=stdin;
   guchar data[500], filename[500];
   guchar *datap=(char *) (&data);
   gint bindex=0, pindex=0, inbut=FALSE;

   sprintf(filename, "%s/.gimp-1.1/gimpbuttonrc", getenv("HOME"));
   if (NULL!=(cf=fopen (filename, "r"))) {
      while (!feof(cf) && bindex < MAX_BUTTONS) {
         if (fscanf(cf, "%[^\n]\n", datap)) {
            if (datap[0] == '<' || datap[0] == '#' || inbut == TRUE) {
               if (pindex >= MAX_ARGS) { g_warning("Too much parameters! Recompile with higher MAX_ARGS"); pindex--; }
               switch (datap[0]) {
               case '<':
                 Buttons[bindex].name = (char *) malloc (strlen(datap+1)*sizeof(char));
                 sscanf(datap+1, " %[^ \t]", Buttons[bindex].name);
                 g_print("\nName: %s\n", Buttons[bindex].name);
                 Buttons[bindex].buttontext = "None specified";
                 Buttons[bindex].nparam = 0;
                 Buttons[bindex].parameters = malloc (sizeof (ProcEntry) * MAX_ARGS);
                 pindex=0;
                 inbut = TRUE;
                 break;
               case 'S':
                 Buttons[bindex].parameters[pindex].data.d_string = (char *) malloc (strlen(datap+1)*sizeof(char));
                 sscanf(datap+1, " %[^\n]\n", Buttons[bindex].parameters[pindex].data.d_string);
                 Buttons[bindex].parameters[pindex].type = PARAM_STRING;
                 g_print("String: \"%s\"\n", Buttons[bindex].parameters[pindex].data.d_string);
                 pindex++;
                 break;
               case 'R': 
                 sscanf(datap+1, " %d", &Buttons[bindex].parameters[pindex].data.d_int32);
                 Buttons[bindex].parameters[pindex].type = PARAM_INT32;
                 g_print("Runmode: %d\n", Buttons[bindex].parameters[pindex].data.d_int32);
                 pindex++;
                 break;
               case 'F': 
                 sscanf(datap+1, " %lf", &Buttons[bindex].parameters[pindex].data.d_float);
                 Buttons[bindex].parameters[pindex].type = PARAM_FLOAT;
                 g_print("Float: %f\n", Buttons[bindex].parameters[pindex].data.d_float);
                 pindex++;
                 break;
               case 'I': 
                 if (data[1] == 'M') {
                   Buttons[bindex].parameters[pindex].type = PARAM_IMAGE;
                   pindex++;
                   g_print ("Image\n"); 
                 } else {
                   sscanf(datap+1, " %d", &Buttons[bindex].parameters[pindex].data.d_int32);
                   Buttons[bindex].parameters[pindex].type = PARAM_INT32;
                   g_print("Integer: %d\n", Buttons[bindex].parameters[pindex].data.d_int32);
                   pindex++;
                 };
                 break;
               case 'D': 
                 Buttons[bindex].parameters[pindex].type = PARAM_DRAWABLE;
                 pindex++;
                 g_print("Drawable\n");
                 break;
               case '>': 
                 Buttons[bindex].buttontext = (char *) malloc (strlen(datap+1)*sizeof(char));
                 sscanf(datap+1, " %[^\n]\n", Buttons[bindex].buttontext);
                 g_print("as Button: %s\n", Buttons[bindex].buttontext);
                 Buttons[bindex].nparam = pindex;
                 bindex++;
                 Buttons[bindex].buttontext = NULL;
                 inbut = FALSE;
                 break;
               case '#': 
                 g_print("Comment: %s\n", data);
                 break;
               default:
                 g_warning("Error in line \"%s\"", datap);
               }
            } else g_warning ("Not in a Button-definition: \"%s\"", datap);
         }
      }
      fclose (cf);

   } else g_warning("Can't open file: %s!\n", filename);
}

         
static gint 
buttons_dialog (void)
{
  GtkWidget *dialog;
  GtkWidget *done_button;
 
  gint i;

  GtkWidget *buttons_vbox;
  GtkWidget *button;

  gint argc = 1;
  gchar **argv = g_new (gchar *, 1);
  argv[0] = g_strdup ("Gimp_Buttons");

  /* Init GTK  */
  gtk_init (&argc, &argv);
  gtk_rc_parse (gimp_gtkrc ());
  gdk_set_use_xshm (gimp_use_xshm ());

  /* Main Dialog */
  dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dialog), PLUG_IN_PRINT_NAME);
  gtk_window_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
  gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
		      (GtkSignalFunc) dialog_close_callback,
		      NULL);
  /*  Action area  */
  done_button = gtk_button_new_with_label ("Close");
  GTK_WIDGET_SET_FLAGS (done_button, GTK_CAN_DEFAULT);
  gtk_signal_connect_object (GTK_OBJECT (done_button), "clicked",
			     (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dialog));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
		      done_button, TRUE, TRUE, 0);
  gtk_widget_grab_default (done_button);
  gtk_widget_show (done_button);


  buttons_vbox = gtk_vbox_new(FALSE,3);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),buttons_vbox);
  gtk_container_border_width (GTK_CONTAINER (buttons_vbox), 4);
  gtk_widget_show(buttons_vbox);

/* HERE GO THE BUTTONS */
  i = 0;
  while (Buttons[i].buttontext != NULL) {
    button = gtk_button_new_with_label (Buttons[i].buttontext);
    Buttons[i].button = button;
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       (GtkSignalFunc) button_clicked_callback,
			       (gpointer) button);
    gtk_container_add(GTK_CONTAINER(buttons_vbox),button);
    gtk_widget_show (button);
    i++;
  }

/* End of Buttons */

  gtk_widget_show(dialog);

  gtk_main ();
  gdk_flush ();

  return FALSE;
}

/* Interface functions  */

static void
dialog_close_callback (GtkWidget *widget,
		       gpointer   data)
{
  gtk_main_quit ();
}

static void
button_clicked_callback (GtkWidget *widget,
		       gpointer   data)
{
  int i, nimages=0, *image_ids=NULL, imagevalid = FALSE;
  gint ActiveDrawable;
  gint bindex = -1;
  int nreturn_vals;
  GParam* return_vals;

  return_vals = gimp_run_procedure ("gimp_image_get_active",
                                    &nreturn_vals,
                                    PARAM_END);

  g_print("\n----------\n");
  if (return_vals[0].data.d_status == STATUS_SUCCESS)
    if ((ActiveImage = return_vals[1].data.d_int32) < 0)  {
      g_print("Currently no image is active!\n");
      imagevalid = FALSE;
    } else {
      g_print("gimp_image_get_active returned %d\n", ActiveImage);
      image_ids = gimp_query_images (&nimages);
      for (i=0; i < nimages; i++) {
        if (image_ids[i] == ActiveImage) { imagevalid = TRUE; break; }
      }
    }

  gimp_destroy_params (return_vals, nreturn_vals);

  if (!imagevalid) {
    gimp_message ("No image active - please click/keypress in target window!");
    return;
  }

  ActiveDrawable = gimp_image_get_active_channel(ActiveImage);
  if (ActiveDrawable == -1) ActiveDrawable = gimp_image_get_active_layer(ActiveImage);

#if 0
#ifdef BUTTONS_DEBUG
  g_print("Gimp_Buttons: waiting... (pid %d)\n", getpid());
  kill(getpid(), 19);
#endif
#endif

  bindex = -1;
  i = 0;
  while (i < MAX_BUTTONS && bindex == -1) {
    if (Buttons[i].button == widget) bindex = i;
    i++;
  }

  for (i=0; i < Buttons[bindex].nparam; i++) {
    if (Buttons[bindex].parameters[i].type == PARAM_IMAGE) Buttons[bindex].parameters[i].data.d_image = ActiveImage;
    if (Buttons[bindex].parameters[i].type == PARAM_DRAWABLE) Buttons[bindex].parameters[i].data.d_drawable = ActiveDrawable;
  }

  g_print("PDB-Call: %s\n", Buttons[bindex].name);
  for (i=0; i < Buttons[bindex].nparam; i++) {
    switch (Buttons[bindex].parameters[i].type) {
    case PARAM_INT32:
      g_print("PARAM_INT32: %d\n", Buttons[bindex].parameters[i].data.d_int32);
      break;
    case PARAM_FLOAT:
      g_print("PARAM_FLOAT: %f\n", Buttons[bindex].parameters[i].data.d_float);
      break;
    case PARAM_STRING:
      g_print("PARAM_STRING: %s\n", Buttons[bindex].parameters[i].data.d_string);
      break;
    case PARAM_IMAGE:
      g_print("PARAM_IMAGE: %d\n", Buttons[bindex].parameters[i].data.d_image);
      break;
    case PARAM_DRAWABLE:
      g_print("PARAM_DRAWABLE: %d\n", Buttons[bindex].parameters[i].data.d_drawable);
      break;
    default:
      g_print("STRANGE THING\n");
    }
  }

  return_vals = gimp_run_procedure2 (Buttons[bindex].name,
     &nreturn_vals,
     Buttons[bindex].nparam,
     Buttons[bindex].parameters);

  if (return_vals[0].data.d_status != STATUS_SUCCESS)
    gimp_message ("Execution failed - Sorry...");

  gimp_displays_flush();

}

